@@ -71,6 +71,7 @@ var (
 	StopIfMod    = false
 	Jobs         sync.WaitGroup
 	TmpsToClean  = make(map[string]struct{})
+	TmpsToCleanM sync.Mutex
 
 	flagTrace          *bool
 	flagTraceAll       *bool
@@ -320,7 +321,11 @@ func runScript(tgt *Tgt, errs chan error, forced, traced bool) error {
 		lockRelease()
 		return TgtError{tgt, ErrLine(err)}
 	}
-	TmpsToClean[fdDep.Name()] = struct{}{}
+	{
+		TmpsToCleanM.Lock()
+		TmpsToClean[fdDep.Name()] = struct{}{}
+		TmpsToCleanM.Unlock()
+	}
 	fdDepOpened := true
 	fdDepExists := true
 	fdDepW := bufio.NewWriter(fdDep)
@@ -330,7 +335,11 @@ func runScript(tgt *Tgt, errs chan error, forced, traced bool) error {
 			fdDep.Close()
 		}
 		if fdDepExists {
-			delete(TmpsToClean, fdDep.Name())
+			{
+				TmpsToCleanM.Lock()
+				delete(TmpsToClean, fdDep.Name())
+				TmpsToCleanM.Unlock()
+			}
 			os.Remove(fdDep.Name())
 		}
 	}
@@ -410,7 +419,11 @@ func runScript(tgt *Tgt, errs chan error, forced, traced bool) error {
 		cleanup()
 		return TgtError{tgt, ErrLine(err)}
 	}
-	TmpsToClean[fdStdout.Name()] = struct{}{}
+	{
+		TmpsToCleanM.Lock()
+		TmpsToClean[fdStdout.Name()] = struct{}{}
+		TmpsToCleanM.Unlock()
+	}
 	fdStdout.Close()
 	tmpPath := fdStdout.Name() + ".3" // and for $3
 	tmpPathRel := mustRel(cwd, tmpPath)
@@ -611,13 +624,19 @@ func runScript(tgt *Tgt, errs chan error, forced, traced bool) error {
 			}
 			lockRelease()
 			if fdDepExists {
-				delete(TmpsToClean, fdDep.Name())
 				os.Remove(fdDep.Name())
 			}
-			delete(TmpsToClean, fdStdout.Name())
 			os.Remove(fdStdout.Name())
-			delete(TmpsToClean, tmpPath)
 			os.Remove(tmpPath)
+			{
+				TmpsToCleanM.Lock()
+				if fdDepExists {
+					delete(TmpsToClean, fdDep.Name())
+				}
+				delete(TmpsToClean, fdStdout.Name())
+				delete(TmpsToClean, tmpPath)
+				TmpsToCleanM.Unlock()
+			}
 			if FdStatus != nil {
 				if _, err = FdStatus.Write([]byte{StatusDone}); err != nil {
 					log.Fatal(err)
@@ -841,7 +860,11 @@ func runScript(tgt *Tgt, errs chan error, forced, traced bool) error {
 		if err != nil {
 			goto Finish
 		}
-		delete(TmpsToClean, fdDep.Name())
+		{
+			TmpsToCleanM.Lock()
+			delete(TmpsToClean, fdDep.Name())
+			TmpsToCleanM.Unlock()
+		}
 		fdDepExists = false
 		if !NoSync {
 			err = ErrLine(syncDir(redoDir))
